from datetime import datetime, timedelta, date
from typing import Union, Tuple, List, Dict
import json

class DateTimeEncoder(json.JSONEncoder):
    
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        if isinstance(obj, date):
            return obj.strftime("%Y-%m-%d")
        
        return json.JSONEncoder.default(self, obj)

def is_later(time1: Union[str, datetime], time2: Union[str, datetime]) -> bool:
    '''
        Return True if time #1 is later than time #2 of time part.
    '''
    _time1: datetime = datetime.strptime(time1, "%Y-%m-%d %H:%M:%S") if isinstance(time1, str) else time1
    _time2: datetime = datetime.strptime(time2, "%Y-%m-%d %H:%M:%S") if isinstance(time2, str) else time2
    
    return _time1.time() > _time2.time()

def datetime2timedelta(_datetime: datetime) -> timedelta:
    return _datetime - datetime(_datetime.year, _datetime.month, _datetime.day, 0, 0, 0)

def is_later_oclock(now_time: datetime, oclock: int) -> bool:
    return datetime2timedelta(now_time) > timedelta(hours=oclock)
    
def is_MorTimeinRange(early_time: int, late_time: int, now_time: datetime) -> bool:
    '''
        判断早安时间是否在范围内
        - early_time: 较早的开始时间
        - late_time: 较晚的结束时间
    '''
    return timedelta(hours=early_time) < datetime2timedelta(now_time) < timedelta(hours=late_time)

def is_NigTimeinRange(early_time: int, late_time: int, now_time: datetime) -> bool:
    '''
        判断晚安时间是否在范围内，注意次日判断
        - early_time: 较早的开始时间
        - late_time: 较晚的结束时间
    '''
    return datetime2timedelta(now_time) > timedelta(hours=early_time) or datetime2timedelta(now_time) < timedelta(hours=late_time)

def total_seconds2tuple_time(secs: int) -> Tuple[int, int, int, int]:
    days: int = secs // (3600 * 24)
    hours: int = (secs - days * 3600 * 24) // 3600
    minutes: int = (secs - days * 3600 * 24 - hours * 3600) // 60
    seconds: int = secs - days * 3600 * 24 - hours * 3600 - minutes * 60
    
    return days, hours, minutes, seconds

def sleeptime_update(_lold: List[int], _sleep: timedelta) -> List[int]:
    '''
        Add a timedelta to another one
        - _lold: days, hrs, mins, secs
    '''
    t_old: timedelta = timedelta(days=_lold[0], hours=_lold[1], minutes=_lold[2], seconds=_lold[3])
    t_new: timedelta = t_old + _sleep
    
    days, hours, minutes, seconds = total_seconds2tuple_time(int(t_new.total_seconds()))
    
    return [days, hours, minutes, seconds]

# A compatible transfer from old version format of data.json into new version's(morning.json)
def morning_json_update(_ofile: Dict[str, Dict[str, Dict[str, int]]]) -> Dict[str, Dict[str, Dict[str, Dict[str, Union[str, int, List[int]]]]]]:
    _nfile: Dict[str, Dict[str, Dict[str, Dict[str, Union[str, int, List[int]]]]]] = dict()
    
    for gid in _ofile:
        # Create groups' info
        _nfile.update({
            gid: {
                "group_count": {
                    "daily": {
                        "good_morning": _ofile[gid]["today_count"]["morning"],
                        "good_night": _ofile[gid]["today_count"]["night"]
                    },
                    "weekly": {
                        "sleeping_king": ""
                    }
                }
            }
        })
        for uid in _ofile[gid]:
            if uid == "today_count":
                continue
            else:
                # Create users' info
                _nfile[gid].update({
                    uid: {
                            "daily": {
                            "morning_time": _ofile[gid][uid]["get_up_time"],
                            "night_time": _ofile[gid][uid]["sleep_time"]
                        },
                        "weekly": {
                            "weekly_morning_count": 0,
                            "weekly_night_count": 0,
                            "weekly_sleep": [0, 0, 0, 0],
                            "lastweek_morning_count": 0,
                            "lastweek_night_count": 0,
                            "lastweek_sleep": [0, 0, 0, 0],
                            "lastweek_earliest_morning_time": _ofile[gid][uid]["get_up_time"],
                            "lastweek_latest_night_time": _ofile[gid][uid]["sleep_time"]
                        },
                        "total": {
                            "morning_count": _ofile[gid][uid]["morning_count"],
                            "night_count": _ofile[gid][uid]["night_count"],
                            "total_sleep": [0, 0, 0, 0]
                        }
                    }
                })
    
    return _nfile


from nonebot import require
from nonebot.adapters.onebot.v11 import MessageSegment
from calendar import MONDAY
from typing import Union, List, Dict, Optional, Tuple
from pathlib import Path
from datetime import datetime, time
import random
from .config import *
from .utils import *
    
require("nonebot_plugin_apscheduler")
from nonebot_plugin_apscheduler import scheduler
from apscheduler.jobstores.base import JobLookupError

class MorningManager:
    def __init__(self):
        self._morning: Dict[str, Dict[str, Dict[str, Dict[str, Union[str, int, datetime, List[int]]]]]] = dict()
        self._morning_path: Path = morning_config.morning_path / "morning.json"
        
        self._config: Dict[str, Dict[str, Dict[str, Union[bool, int]]]] = dict()
        self._config_path: Path = morning_config.morning_path / "config.json"

    def _init_group_data(self, gid: str) -> None:
        '''
            Initialize group data.
        '''
        self._load_data()
        if gid not in self._morning:
            self._morning.update({
                gid: {
                    "group_count": {
                        "daily": {
                            "good_morning": 0,  # Daily good-morning count of groups, REFRESH at the late time of good-morning everyday
                            "good_night": 0     # Daily good-night count of groups, REFRESH at the late time of good-night everyday
                        },
                        "weekly": {
                            "sleeping_king": "" # Sleeping king of group of last week, REFRESH at the late time of good-morning every Monday
                        }
                    }
                }
            })
            
            self._save_data()

    def get_current_config(self) -> MessageSegment:
        '''
            Return the current configurations.
        '''
        msg: str = "早安晚安设置如下："
        self._load_config()
        
        # Morning config
        morning_intime: bool = self._config["morning"]["morning_intime"]["enable"]
        if morning_intime:
            msg += "\n是否要求规定时间内起床：是\n - 最早允许起床时间：" + str(self._config["morning"]['morning_intime']["early_time"]) + "点\n - 最晚允许起床时间：" + str(self._config["morning"]["morning_intime"]["late_time"]) + "点"
        else:
            msg += "\n是否要求规定时间内起床：否"
            
        multi_get_up: bool = self._config["morning"]["multi_get_up"]["enable"]
        if multi_get_up:
            msg += "\n是否允许连续多次起床：是"
        else:
            msg += "\n是否允许连续多次起床：否\n - 允许的最短起床间隔：" + str(self._config["morning"]["multi_get_up"]["interval"]) + "小时"
        
        super_get_up: bool = self._config["morning"]["super_get_up"]["enable"]
        if super_get_up:
            msg += "\n是否允许超级亢奋(即睡眠时长很短)：是"
        else:
            msg += "\n是否允许超级亢奋(即睡眠时长很短)：否\n - 允许的最短睡觉时长：" + str(self._config["morning"]["super_get_up"]["interval"]) + "小时"
        
        # Night config
        night_intime: bool = self._config["night"]["night_intime"]["enable"]
        if night_intime:
            msg += "\n是否要求规定时间内睡觉：是\n - 最早允许睡觉时间：" + str(self._config["night"]["night_intime"]["early_time"]) + \
                "点\n - 最晚允许睡觉时间：第二天早上" + str(self._config["night"]["night_intime"]["late_time"]) + "点"
        else:
            msg += "\n是否要求规定时间内睡觉：否"
        
        good_sleep: bool = self._config["night"]["good_sleep"]["enable"]
        if good_sleep:
            msg += "\n是否开启优质睡眠：是"
        else:
            msg += "\n是否开启优质睡眠：否\n - 允许的最短优质睡眠：" + str(self._config["night"]["good_sleep"]["interval"]) + "小时"
        
        deep_sleep: bool = self._config["night"]["deep_sleep"]["enable"]
        if deep_sleep:
            msg += "\n是否允许深度睡眠(即清醒时长很短)：是"
        else:
            msg += "\n是否允许深度睡眠(即清醒时长很短)：否\n - 允许的最短清醒时长：" + str(self._config["night"]["deep_sleep"]["interval"]) + "小时"
        
        return MessageSegment.text(msg)

    # ------------------------------ Config ------------------------------ #
    def _change_enable(self, day_or_night: str, _setting: str, new_state: bool) -> str:
        '''
            Change and save the new state of a setting.
        '''
        self._load_config()
        self._config[day_or_night][_setting]["enable"] = new_state
        self._save_config()

        return "配置更新成功！"

    def _change_set_time(self, _day_or_night: str, _setting: str, _interval_or_early_time: int, _late_time: Optional[int] = None) -> str:
        '''
            Change the interval of a setting.
        '''
        self._load_config()
        
        if _setting == "morning_intime" or _setting == "night_intime":
            early_time: int = _interval_or_early_time
            if isinstance(_late_time, int):
                late_time: int = _late_time
            else:
                return "配置更新失败：缺少参数！"
            
            self._config[_day_or_night][_setting]["early_time"] = early_time
            self._config[_day_or_night][_setting]["late_time"] = late_time
        else:
            interval: int = _interval_or_early_time
            self._config[_day_or_night][_setting]["interval"] = interval
        
        msg: str = "配置更新成功！"
        
        # Some settings are True in default
        if _setting == "morning_intime" or _setting == "night_intime" or _setting == "good_sleep" \
            and self._config[_day_or_night][_setting]["enable"] == False:
            self._config[_day_or_night][_setting]["enable"] = True
            msg += "且此项设置已启用！"
        
        # Some settings are False in default
        if _setting == "multi_get_up" or _setting == "super_get_up" or _setting == "deep_sleep" \
            and self._config[_day_or_night][_setting]["enable"] == True:
            self._config[_day_or_night][_setting]["enable"] = False
            msg += "且此项设置已禁用！"
        
        self._save_config()
            
        return msg
        
    def daily_refresh(self) -> None:
        '''
            Reset good-morning/night count of groups of yesterday at the earliest time of daily good-night.
        '''
        self._load_data()
        
        for gid in self._morning:
            self._morning[gid]["group_count"]["daily"]["good_morning"] = 0
            self._morning[gid]["group_count"]["daily"]["good_night"] = 0
        
        self._save_data()
        logger.info("每日早晚安已刷新！")
        
    def weekly_night_refresh(self) -> None:
        '''
            1. Refresh good-night count of last week at 0 A.M. every Monday.
            2. Reset weekly good-night count at 0 A.M. every Monday.
        '''
        self._load_data()
        
        for gid in self._morning:
            for uid, user_items in self._morning[gid].items():
                # Remember to jump over the key "group_count"
                if uid == "group_count":
                    continue
                
                user_items["weekly"]["lastweek_night_count"] = user_items["weekly"]["weekly_night_count"]
                user_items["weekly"]["weekly_night_count"] = 0
                
        self._save_data()
        
    def weekly_sleep_time_refresh(self) -> None:
        '''
            1. Refresh sleeping time & good-morning count of last week.
            2. Refresh the sleeping king UID of each groups.
            3. Reset weekly sleeping time & good-morning count.
        '''
        self._load_data()
        
        for gid in self._morning:
            _max_sleep_time: List[int] = [0, 0, 0, 0]
            _sleeping_king_uid: str = ""
            
            for uid, user_items in self._morning[gid].items():
                # Remember to jump over the key "group_count"
                if uid == "group_count": 
                    continue
                
                user_items["weekly"]["lastweek_morning_count"] = user_items["weekly"]["weekly_morning_count"]
                user_items["weekly"]["lastweek_sleep"] = user_items["weekly"]["weekly_sleep"]
                
                user_items["weekly"]["weekly_morning_count"] = 0
                user_items["weekly"]["weekly_sleep"] = [0, 0, 0, 0]
                
                # Compare sleeping times, day > hrs > mins > secs
                if user_items["weekly"]["lastweek_sleep"] > _max_sleep_time:
                    _max_sleep_time = user_items["weekly"]["lastweek_sleep"]
                    _sleeping_king_uid = uid
                    
            self._morning[gid]["group_count"]["weekly"]["sleeping_king"] = _sleeping_king_uid
                
        self._save_data()
        logger.info("每周睡眠时间、每周早安已刷新！")

    def morning_config(self, _mor_setting: str, *args: List[int]) -> MessageSegment:
        '''
            Configurations about morning.
        '''
        _setting: str = mor_switcher[_mor_setting]
        if _setting == "morning_intime":
            early_time: int = args[0]
            late_time: int = args[1]
            
            if early_time < 0 or early_time > 24 or late_time < 0 or late_time > 24:
                msg = "错误！您设置的时间未在0-24之间，要求：0 <= 时间 <= 24"
            else:
                msg = self._change_set_time("morning", _setting, early_time, late_time)
            
                # Change the data of daily good-morning/night scheduler and weekly sleeping time scheduler
                self.daily_scheduler(early_time)
                self.weekly_sleep_time_scheduler(late_time)
        else:
            interval: int = args[0]
            
            if interval < 0 or interval > 24:
                msg = "错误！您设置的时间间隔未在0-24之间，要求：0 <= 时间 <= 24"
            else:
                msg = self._change_set_time("morning", _setting, interval)
        
        return MessageSegment.text(msg)

    def morning_switch(self, _mor_setting: str, new_state: bool) -> MessageSegment:
        '''
            Enable/Disable of morning settings.
        '''
        _setting: str = mor_switcher[_mor_setting]
        msg: str = self._change_enable("morning", _setting, new_state)
        
        # Change the status of daily good-morning/night schedulers and weekly sleeping time scheduler
        if _setting == "morning_intime":
            # Remove the schedulers
            if not new_state:
                try:
                    scheduler.remove_job("weekly_sleep_time_scheduler")
                    logger.info(f"每周睡眠时间定时刷新任务已关闭！")
                    
                except JobLookupError as e:
                    logger.warning(f"每周睡眠时间定时刷新任务移除失败: {e}")
                    msg += "\n每周睡眠时间定时刷新任务移除失败"
                
                try:
                    scheduler.remove_job("daily_scheduler")
                    logger.info(f"每日早晚安定时刷新任务已关闭！")
                    
                except JobLookupError as e:
                    logger.warning(f"每日早晚安定时刷新任务移除失败: {e}")
                    msg += "\n每日早晚安定时刷新任务移除失败"
            
            # Add the schedulers if they don't exist
            else:
                if not scheduler.get_job("weekly_sleep_time_scheduler"):
                    self.weekly_sleep_time_scheduler()
                    logger.info("每周睡眠时间定时刷新任务已启动！")
                
                if not scheduler.get_job("daily_scheduler"):      
                    self.daily_scheduler()
                    logger.info("每日早晚安定时刷新任务已启动！")
        
        return MessageSegment.text(msg)

    def night_config(self, _nig_setting: str, *args: List[int]) -> MessageSegment:
        '''
            Configurations about night.
        '''
        _setting: str = nig_switcher[_nig_setting]
        if _setting == "night_intime":
            early_time: int = args[0]
            late_time: int = args[1]
            
            if early_time < 0 or early_time > 24 or late_time < 0 or late_time > 24:
                msg = "错误！您设置的时间未在0-24之间，要求：0 <= 时间 <= 24"
            else:
                msg = self._change_set_time("night", _setting, early_time, late_time)
        else:
            interval: int = args[0]
            
            if interval < 0 or interval > 24:
                msg = "错误！您设置的时间间隔未在0-24之间，要求：0 <= 时间 <= 24"
            else:
                msg = self._change_set_time("night", _setting, interval, None)
        
        return MessageSegment.text(msg)

    def night_switch(self, _nig_setting: str, new_state: bool) -> MessageSegment:
        '''
            Enable/Disable of night settings.
        '''
        _setting: str = nig_switcher[_nig_setting]
        msg: str = self._change_enable("night", _setting, new_state)
        
        return MessageSegment.text(msg)

    # ------------------------------ Morning Judgement ------------------------------ #
    def _morning_and_update(self, gid: str, uid: str, now_time: datetime) -> Tuple[str, Union[str, int]]:
        '''
            Morning & update data.
        '''
        self._load_data()
        
        # 起床并写数据
        sleep_time: datetime = datetime.strptime(self._morning[gid][uid]["daily"]["night_time"], "%Y-%m-%d %H:%M:%S")
        in_sleep: timedelta = now_time - sleep_time
        days, hours, minutes, seconds = total_seconds2tuple_time(int(in_sleep.total_seconds()))
        
        # 睡觉时间小于24小时就同时给出睡眠时长，记录；否则隔日
        in_sleep_tmp: str = ""
        
        if in_sleep.days > 0:
            in_sleep_tmp = ""
        else:
            in_sleep_tmp = f"{hours}时{minutes}分{seconds}秒"
            self._morning[gid][uid]["weekly"]["weekly_sleep"] = sleeptime_update(self._morning[gid][uid]["weekly"]["weekly_sleep"], in_sleep)
            self._morning[gid][uid]["total"]["total_sleep"] = sleeptime_update(self._morning[gid][uid]["total"]["total_sleep"], in_sleep)

        # Daily morning time
        self._morning[gid][uid]["daily"]["morning_time"] = now_time
        # Weekly morning count add
        self._morning[gid][uid]["weekly"]["weekly_morning_count"] += 1
        # Total morning count add
        self._morning[gid][uid]["total"]["morning_count"] += 1
        
        lastweek_emt: Union[int, str] = self._morning[gid][uid]["weekly"]["lastweek_earliest_morning_time"]
        
        # If it is integer, it means no data
        if isinstance(lastweek_emt, int):
            self._morning[gid][uid]["weekly"]["lastweek_earliest_morning_time"] = now_time
        else:
            # If weekly morning time is later than daily's, update
            if not is_later(self._morning[gid][uid]["daily"]["morning_time"], self._morning[gid][uid]["weekly"]["lastweek_earliest_morning_time"]):
                self._morning[gid][uid]["weekly"]["lastweek_earliest_morning_time"] = now_time
        
        # 判断是今天第几个起床的
        self._morning[gid]["group_count"]["daily"]["good_morning"] += 1
        
        self._save_data()

        return self._morning[gid]["group_count"]["daily"]["good_morning"], in_sleep_tmp if in_sleep_tmp != "" else 0

    def get_morning_msg(self, gid: str, uid: str, sex_str: str) -> MessageSegment:
        '''
            Return good-morning info.
        '''
        self._load_config()
        
        # 若开启规定时间早安，则判断该时间是否允许早安
        now_time: datetime = datetime.now()
        if self._config["morning"]["morning_intime"]["enable"]:
            _early_time: int = self._config["morning"]["morning_intime"]["early_time"]
            _late_time: int = self._config["morning"]["morning_intime"]["late_time"]
            
            if not is_MorTimeinRange(_early_time, _late_time, now_time):
                msg = f"现在不能早安哦，可以早安的时间为{_early_time}时到{_late_time}时~"
                return MessageSegment.text(msg)

        self._init_group_data(gid)
        
        # 当数据里有过这个人的信息
        if uid in self._morning[gid]:
            # 判断是否隔日
            last_sleep_time: datetime = datetime.strptime(self._morning[gid][uid]["daily"]["night_time"], "%Y-%m-%d %H:%M:%S")
            if last_sleep_time - now_time < timedelta(hours=24):
            
                # 若关闭连续多次早安，则判断在设定时间内是否多次早安
                if not self._config["morning"]["multi_get_up"]["enable"] and self._morning[gid][uid]["daily"]["morning_time"] != 0:
                    interval: int = self._config["morning"]["multi_get_up"]["interval"]
                    morning_time: datetime = datetime.strptime(self._morning[gid][uid]["daily"]["morning_time"], "%Y-%m-%d %H:%M:%S")
                    
                    if now_time - morning_time < timedelta(hours=interval):
                        msg = f"{interval}小时内你已经早安过了哦~"
                        return MessageSegment.text(msg)
                
                # 若关闭超级亢奋，则判断睡眠时长是否小于设定时间
                if not self._config["morning"]["super_get_up"]["enable"]:
                    interval: int = self._config["morning"]["super_get_up"]["interval"]
                    night_time: datetime = datetime.strptime(self._morning[gid][uid]["daily"]["night_time"], "%Y-%m-%d %H:%M:%S")
                    
                    if now_time - night_time < timedelta(hours=interval):
                        msg = "你可猝死算了吧？现在不能早安哦~"
                        return MessageSegment.text(msg)
            # 有信息但是隔日
            else:
                msg = random.choice(morning_prompt)
                return MessageSegment.text(msg)
                  
        # 否则说明：他还没睡过觉；即便如此，还是回复早安！
        else:
            msg = random.choice(morning_prompt)
            return MessageSegment.text(msg)
            
        # 当前面条件均符合的时候，允许早安
        num, in_sleep = self._morning_and_update(gid, uid, now_time)
        if isinstance(in_sleep, str):
            msg = f"早安成功！你的睡眠时长为{in_sleep}，\n你是今早第{num}个起床的{sex_str}！"
        else:
            msg = random.choice(morning_prompt)
 
        return MessageSegment.text(msg)

    # ------------------------------ Night Judgement ------------------------------ #
    def _night_and_update(self, gid: str, uid: str, now_time: datetime) -> Tuple[str, Union[str, int]]:
        '''
            Good night & update.
        '''
        self._load_data()
        
        # 没有晚安数据，则创建
        if uid not in self._morning[gid]:
            self._morning[gid].update({
                uid: {
                    "daily": {
                        "morning_time": 0, 
                        "night_time": now_time
                    },
                    "weekly": {
                        "weekly_morning_count": 0,          # Weekly good-morning count,                RESET at the late time of good-morning every Monday
                        "weekly_night_count": 1,            # Weekly good-night count,                  RESET at 0 A.M. every Monday
                        "weekly_sleep": [0, 0, 0, 0],       # Weekly sleeping time, list of days/hrs/mins/secs, RESET at the late time of good-morning every Monday
                        "lastweek_morning_count": 0,        # Good-morning count of last week,          REFRESH at the late time of good-morning every Monday
                        "lastweek_night_count": 0,          # Good-night count of last week,            REFRESH at 0 A.M. every Monday
                        "lastweek_sleep": [0, 0, 0, 0],     # Sleeping time of last week, list of days/hrs/mins/secs, REFRESH at the late time of good-morning every Monday
                        "lastweek_earliest_morning_time": 0,# Earliest good-morning time of last week,  REFRESH at a new daily good-morning time in
                        "lastweek_latest_night_time": 0     # Latest good-night time of last week,      REFRESH at a new daily good-night time in
                    },
                    "total": {
                        "morning_count": 0,                 # Total good-morning count, never RESET
                        "night_count": 1,                   # Total good-night count, never RESET
                        "total_sleep": [0, 0, 0, 0]         # Total sleeping time, list of days/hrs/mins/secs, REFRESH at every valid good-morning
                    }
                }
            })
            
            self._morning[gid][uid]["weekly"]["lastweek_latest_night_time"] = self._morning[gid][uid]["daily"]["night_time"]
            
        # 若有就更新数据
        else:
            # Daily night time
            self._morning[gid][uid]["daily"]["night_time"] = now_time
            # Weekly night count add
            self._morning[gid][uid]["weekly"]["weekly_night_count"] += 1
            # Total night count add
            self._morning[gid][uid]["total"]["night_count"] += 1
            
            lastweek_lnt: Union[int, str] = self._morning[gid][uid]["weekly"]["lastweek_latest_night_time"]
        
            # If lastweek_lnt is integer, it means no data
            if isinstance(lastweek_lnt, int):
                self._morning[gid][uid]["weekly"]["lastweek_latest_night_time"] = now_time
            else:
                # If daily sleep time is later than weekly's, update
                if is_later(self._morning[gid][uid]["daily"]["night_time"], self._morning[gid][uid]["weekly"]["lastweek_latest_night_time"]):
                    self._morning[gid][uid]["weekly"]["lastweek_latest_night_time"] = now_time
          
        # 当上次起床时间非0，计算清醒时长
        in_day_tmp: str = ""
        if self._morning[gid][uid]["daily"].get("morning_time", 0) != 0:
            get_up_time: datetime = datetime.strptime(self._morning[gid][uid]["daily"]["morning_time"], "%Y-%m-%d %H:%M:%S")
            in_day: timedelta = now_time - get_up_time
            _, hours, minutes, seconds = total_seconds2tuple_time(int(in_day.total_seconds()))
            
            if in_day.days > 0:
                in_day_tmp = ""
            else:
                in_day_tmp = f"{hours}时{minutes}分{seconds}秒"

        # 判断是今天第几个睡觉的
        self._morning[gid]["group_count"]["daily"]["good_night"] += 1
        
        self._save_data()

        return self._morning[gid]["group_count"]["daily"]["good_night"], in_day_tmp if in_day_tmp != "" else 0

    def get_night_msg(self, gid: str, uid: str, sex_str: str) -> MessageSegment:
        '''
            Return good-night info.
        '''
        self._load_config()
        
        # 若开启规定时间晚安，则判断该时间是否允许晚安
        now_time: datetime = datetime.now()
        if self._config["night"]["night_intime"]["enable"]:
            _early_time: int = self._config["night"]["night_intime"]["early_time"]
            _late_time: int = self._config["night"]["night_intime"]["late_time"]
            
            if not is_NigTimeinRange(_early_time, _late_time, now_time):
                msg = f"现在不能晚安哦，可以晚安的时间为{_early_time}时到第二天早上{_late_time}时~"
                return MessageSegment.text(msg)

        self._init_group_data(gid)

        # 当数据里有过这个人的信息就判断:
        if uid in self._morning[gid]:
            
            # 若开启优质睡眠，则判断在设定时间内是否多次晚安
            if self._config["night"]["good_sleep"]["enable"]:
                interval: int = self._config["night"]["good_sleep"]["interval"]
                night_time: datetime = datetime.strptime(self._morning[gid][uid]["daily"]["night_time"], "%Y-%m-%d %H:%M:%S")
                
                if now_time - night_time < timedelta(hours=interval):
                    msg = f"{interval}小时内你已经晚安过了哦~"
                    return MessageSegment.text(msg)
            
            # 若关闭深度睡眠，则判断不在睡觉的时长是否小于设定时长
            if isinstance(self._morning[gid][uid]["daily"]["morning_time"], str):
                if not self._config["night"]["deep_sleep"]["enable"]:
                    interval: int = self._config["night"]["deep_sleep"]["interval"]
                    morning_time: datetime = datetime.strptime(self._morning[gid][uid]["daily"]["morning_time"], "%Y-%m-%d %H:%M:%S")
                    
                    if now_time - morning_time < timedelta(hours=interval):
                        msg = "睡这么久还不够？现在不能晚安哦~"
                        return MessageSegment.text(msg)

        # 当数据里没有这个人或者前面条件均符合的时候，允许晚安
        num, in_day = self._night_and_update(gid, uid, now_time)
        if isinstance(in_day, int):
            msg = f"晚安成功！你是今晚第{num}个睡觉的{sex_str}！"
        else:
            msg = f"晚安成功！你今天的清醒时长为{in_day}，\n你是今晚第{num}个睡觉的{sex_str}！"
            
        return MessageSegment.text(msg)
    
    # ------------------------------ Routine ------------------------------ #
    def get_my_routine(self, gid: str, uid: str) -> MessageSegment:
        '''
            Get user's routine.
            
            If on Monday and now is later than the latest time of good-morning of Monday, good-morning/night count of last week & sleeping time will be included.
            
            Else, add weekly info of current week.
        '''
        self._init_group_data(gid)
        
        now_time: datetime = datetime.now()
        today: int = now_time.weekday()
        
        if uid in self._morning[gid]:
            # Daily info
            get_up_time: str = self._morning[gid][uid]["daily"]["morning_time"]
            sleep_time: str = self._morning[gid][uid]["daily"]["night_time"]
            
            # Total info
            morning_count: int = self._morning[gid][uid]["total"]["morning_count"]
            night_count: int = self._morning[gid][uid]["total"]["night_count"]
            total_sleep: List[int] = self._morning[gid][uid]["total"]["total_sleep"]
            
            msg: str = "你的作息数据如下："
            msg += f"\n最近一次早安时间为{get_up_time}"
            msg += f"\n最近一次晚安时间为{sleep_time}"
            
            week_list: List[str] = ["一", "二", "三", "四", "五", "六", "日"]
                
            # When on Monday and now time is later than the latest time of good-morning
            if today == MONDAY:
                threshold_hour: int = self.get_refresh_time("morning", "late_time")
                
                if threshold_hour != -1 and is_later_oclock(now_time, threshold_hour):
                    lastweek_morning_count: int = self._morning[gid][uid]["weekly"]["lastweek_morning_count"]
                    lastweek_night_count: int = self._morning[gid][uid]["weekly"]["lastweek_night_count"]
                    lastweek_sleep: List[int] = self._morning[gid][uid]["weekly"]["lastweek_sleep"]
                    
                    lastweek_lnt_date: datetime = datetime.strptime(self._morning[gid][uid]["weekly"]["lastweek_latest_night_time"], "%Y-%m-%d %H:%M:%S")
                    lastweek_lnt: time = lastweek_lnt_date.time()
                    latest_day: int = lastweek_lnt_date.weekday()
                    
                    lastweek_emt_date: datetime = datetime.strptime(self._morning[gid][uid]["weekly"]["lastweek_earliest_morning_time"], "%Y-%m-%d %H:%M:%S")
                    lastweek_emt: time = lastweek_emt_date.time()
                    earliest_day: int = lastweek_emt_date.weekday()
                    
                    msg += f"\n上周早安了{lastweek_morning_count}次"
                    msg += f"\n上周晚安了{lastweek_night_count}次"
                    msg += f"\n上周睡眠时间为{lastweek_sleep[0]}天{lastweek_sleep[1]}时{lastweek_sleep[2]}分{lastweek_sleep[3]}秒"
                    msg += f"\n上周最晚晚安时间是周{week_list[latest_day]} {lastweek_lnt}"
                    msg += f"\n上周最早早安时间是周{week_list[earliest_day]} {lastweek_emt}"
            
            # Not on Monday, add weekly info
            else:
                weekly_morning_count: int = self._morning[gid][uid]["weekly"]["weekly_morning_count"]
                weekly_night_count: int = self._morning[gid][uid]["weekly"]["weekly_night_count"]
                
                msg += f"\n本周早安了{weekly_morning_count}次"
                msg += f"\n本周晚安了{weekly_night_count}次"
                
            msg += f"\n一共早安了{morning_count}次"
            msg += f"\n一共晚安了{night_count}次"
            msg += f"\n一共睡眠了{total_sleep[0]}天{total_sleep[1]}时{total_sleep[2]}分{total_sleep[3]}秒"
        else:
            msg = "你本周还没有早晚安过呢！暂无数据~"
        
        return MessageSegment.text(msg)

    def get_group_routine(self, gid: str) -> Tuple[int, int, Optional[str]]:
        '''
            Get group's routine: daily good-morning/night count.
            If on Monday and now is later than the latest time of good-morning of Monday, add sleeping king of last week.
        '''
        self._init_group_data(gid)
        
        now_time: datetime = datetime.now()
        today: int = now_time.weekday()
        
        morning_count: int = self._morning[gid]["group_count"]["daily"]["good_morning"]
        night_count: int = self._morning[gid]["group_count"]["daily"]["good_night"]        
        
        if today == MONDAY:
            uid: str = ""
            threshold_hour: int = self.get_refresh_time("morning", "late_time")
            
            if threshold_hour != -1 and is_later_oclock(now_time, threshold_hour):
                uid = self._morning[gid]["group_count"]["weekly"]["sleeping_king"]
            
            return morning_count, night_count, uid if uid != "" else None
        
        return morning_count, night_count, None

    # ------------------------------ Utils ------------------------------ #
    def _save_data(self) -> None:
        with open(self._morning_path, 'w', encoding='utf-8') as f:
            json.dump(self._morning, f, ensure_ascii=False, indent=4, cls=DateTimeEncoder)
                
    def _save_config(self) -> None:
        with open(self._config_path, 'w', encoding='utf-8') as f:
            json.dump(self._config, f, ensure_ascii=False, indent=4)

    def _load_data(self) -> None:
        with open(self._morning_path, "r", encoding="utf-8") as f:
            self._morning = json.load(f)
        
    def _load_config(self) -> None:
        with open(self._config_path, "r", encoding="utf-8") as f:
            self._config = json.load(f)
            
    def get_refresh_time(self, day_or_night: str, key: str) -> int:
        '''
            Get the time given the specific type(day or night) and key.
        '''
        self._load_config()
        
        if day_or_night == "morning":
            return self._config[day_or_night]["morning_intime"][key] if self._config[day_or_night]["morning_intime"]["enable"] else -1

        if day_or_night == "night":
            return self._config[day_or_night]["night_intime"][key] if self._config[day_or_night]["night_intime"]["enable"] else -1
        
    def daily_scheduler(self, hours: Optional[int] = None) -> None:
        '''
            Run the scheduler for refreshing daily good-morning/night counts. Replace the existing scheduler.
        '''
        if isinstance(hours, int):
            _hours = hours
        else:
            _hours = self.get_refresh_time("night", "early_time")
        
        if _hours != -1:
            scheduler.add_job(
                self.daily_refresh,
                "cron",
                id="daily_scheduler",
                replace_existing=True,
                hour=_hours,
                minute=0,
                misfire_grace_time=60
            )

    def weekly_sleep_time_scheduler(self, hours: Optional[int] = None) -> None:
        '''
            Run the scheduler for refreshing the weekly sleeping time. Replace the existing scheduler.
        '''
        if isinstance(hours, int):
            _hours = hours
        else:
            _hours = self.get_refresh_time("morning", "late_time")
        
        if _hours != -1:
            scheduler.add_job(
                self.weekly_sleep_time_refresh,
                "cron",
                id="weekly_sleep_time_scheduler",
                replace_existing=True,
                hour=_hours,
                minute=0,
                day_of_week="0",    # From Monday to Sunday: 0~6 
                misfire_grace_time=60
            )

morning_manager = MorningManager()

__all__ = [
    morning_manager
]


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nonebot_plugin_morning']

package_data = \
{'': ['*'], 'nonebot_plugin_morning': ['resource/*']}

install_requires = \
['nonebot-adapter-onebot>=2.1.1,<3.0.0',
 'nonebot-plugin-apscheduler>=0.1.3,<0.2.0',
 'nonebot2>=2.0.0b3,<3.0.0']

setup_kwargs = {
    'name': 'nonebot-plugin-morning',
    'version': '0.3.1',
    'description': 'Good morning & good night!',
    'long_description': '<div align="center">\n\n# Good Morning\n\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable-next-line MD036 -->\n_🌈 おはよう！ 🌈_\n<!-- prettier-ignore-end -->\n\n</div>\n<p align="center">\n  \n  <a href="https://github.com/MinatoAquaCrews/nonebot_plugin_morning/blob/beta/LICENSE">\n    <img src="https://img.shields.io/github/license/MinatoAquaCrews/nonebot_plugin_morning?color=blue">\n  </a>\n  \n  <a href="https://github.com/nonebot/nonebot2">\n    <img src="https://img.shields.io/badge/nonebot2-2.0.0b3+-green">\n  </a>\n  \n  <a href="https://github.com/MinatoAquaCrews/nonebot_plugin_morning/releases/tag/v0.3.1">\n    <img src="https://img.shields.io/github/v/release/MinatoAquaCrews/nonebot_plugin_morning?color=orange&include_prereleases">\n  </a>\n\n  <a href="https://www.codefactor.io/repository/github/MinatoAquaCrews/nonebot_plugin_morning">\n    <img src="https://img.shields.io/codefactor/grade/github/MinatoAquaCrews/nonebot_plugin_morning/beta?color=red">\n  </a>\n  \n</p>\n\n## 版本\n\nv0.3.1\n\n⚠ 适配nonebot2-2.0.0beta.3+\n\n[更新日志](https://github.com/MinatoAquaCrews/nonebot_plugin_morning/releases/tag/v0.3.1)\n\n## 安装\n\n1. 通过`pip`或`nb`安装；\n\n2. 用户数据`data.json`及早晚安配置文件`config.json`位于`./resource`下，可在`env`内设置`MORNING_PATH`更改：\n\n    ``` python\n    MORNING_PATH="path-to-your-resource"    # For example: ./my_data/morning_resource/\n    ```\n\n## 功能\n\n1. 和Bot说早晚安，记录睡眠时间，培养良好作息；\n\n2. 群管及超管可设置早安时限、晚安时限、优质睡眠、深度睡眠等，参见规则配置；\n\n3. 🔥 **重磅更新** 分群管理群友作息，群友的早晚安数据会记录至`morning.json`内，形如：\n\n    ``` python\n    {\n        "123456789": {                                      # 群号\n            "group_count": {                                # 群统计\n                "daily": {\n                    "good_morning": 1,                      # 群每日早安人数\n                    "good_night": 1                         # 群每日晚安人数\n                },\n                "weekly": {\n                    "sleeping_king": "123456"               # 群上周睡觉大王\n                }\n            },\n            # 群友个人作息统计\n            "123456": {                                     # 群友QQ号\n                "daily": {\n                    "morning_time": "2022-01-01 07:08:09",  # 每日早安时间\n                    "night_time": "2022-01-01 22:33:44"     # 每日晚安时间\n                },\n                "weekly": {\n                    "weekly_morning_count": 1,              # 周早安天数\n                    "weekly_night_count": 1,                # 周晚安天数\n                    "weekly_sleep": [0, 7, 8, 9],           # 周睡眠时长，列表形式：天/时/分/秒，例如0天7时8分9秒\n                    "lastweek_morning_count": 1,            # 上周早安天数（暂存）\n                    "lastweek_night_count": 1,              # 上周晚安天数（暂存）\n                    "lastweek_sleep": [0, 7, 8, 9],         # 上周睡眠时长（暂存）\n                    "lastweek_earliest_morning_time": 0,    # 上周早安最早的时间（暂存）\n                    "lastweek_latest_night_time": 0         # 上周晚安最晚的时间（暂存）\n                },\n                "total": {\n                    "morning_count": 1,                     # 总早安次数\n                    "night_count": 1,                       # 总晚安次数\n                    "total_sleep": [0, 7, 8, 9]             # 总睡眠时间\n                }\n            }       \n        }\n    }\n    ```\n\n    👉 说明：\n    \n    1. 个人信息字段的`daily`储存每日作息数据；`weekly`储存每周作息数据，`lastweek_`前缀的数据为上周的“每周作息数据”，即暂存上周作息数据；`total`为总作息数据；\n\n    2. 在**每周一的最晚早安时间**定时更新，在周一且当天**最晚早安时间**后，`群友作息`与`我的作息`将增加有关上周作息数据的信息（包括`lastweek_`前缀的数据与群信息字段中的`weekly`数据，目前仅有上周睡觉大王）；\n\n4. **新增** 将自动更新`v0.2.x`的数据文件`data.json`为新版`morning.json`；将自动更新`v0.2.x`的配置文件`config.json`部分键值。在插件初始化时，若不存在新版数据文件且存在旧版数据文件`data.json`，则更新数据字段、保留原始数据，否则创建空的新版数据文件。\n   \n   ⚠ 若存在新版数据文件`morning.json`，则不会重复更新\n\n   ⚠ 自动检测配置文件`config.json`的部分键值是否为旧版并自动更新，后续版本将弃用该部分兼容\n\n5. おはよう！🌈\n\n## 命令\n\n1. 早晚安：[早安/晚安]，记录睡眠时间；\n\n2. **新增** 查看我的作息：[我的作息]：每周一新增显示上周睡眠信息统计，其他时间显示本周信息；\n\n3. **新增** 查看群友作息：[群友作息]，看看今天几个群友睡觉或起床了，每周一新增显示上周睡觉王；\n\n4. 查看当前安早晚安配置（规则）：[早晚安设置]；\n\n5. [管理员或超管] 设置命令\n\n    - 开启/关闭某个配置：早安/晚安开启/关闭某项功能；\n\n    - 早安/晚安设置：设置功能的参数；\n\n    - 详见规则配置；\n\n## 规则配置\n\n`confg.json`文件已默认写入下述配置，会自动检测旧版配置文件并自动更新，当其不存在时则创建并写入下述**初始值**：\n\n``` python\n{\n    "morning": {\n        "morning_intime": {         # 是否只能在规定时间起床\n            "enable": true,         # 默认开启，若关闭则下面两项无效\n            "early_time": 6,        # 允许的最早的起床时间，默认早上6点\n            "late_time": 12         # 允许的最晚的起床时间，默认中午12点\n        },\n        "multi_get_up": {           # 是否允许多次起床\n            "enable": false,        # 默认不允许，若允许则下面一项无效\n            "interval": 6           # 两次起床间隔的时间，小于这个时间就不允许起床\n        },\n        "super_get_up": {           # 是否允许超级亢奋，即睡眠时长很短\n            "enable": false,        # 默认不允许，若允许则下面一项无效\n            "interval": 3           # 这次起床和上一次睡觉的时间间隔，小于这个时间就不允许起床，不怕猝死？给我睡！\n        }\n    },\n    "night": {\n        "night_intime": {           # 是否只能在规定时间睡觉\n            "enable": true,         # 默认开启，若关闭则下面两项无效\n            "early_time": 21,       # 允许的最早的睡觉时间，默认晚上21点\n            "late_time": 6          # 允许的最晚的睡觉时间，默认次日早上6点\n        },\n        "good_sleep": {             # 是否开启优质睡眠\n            "enable": true,         # 默认开启，若关闭则下面一项无效\n            "interval": 6           # 两次睡觉间隔的时间，小于这个时间就不允许睡觉\n        },\n        "deep_sleep": {             # 是否允许深度睡眠，即清醒时长很短\n            "enable": false,        # 默认不允许，若允许则下面一项无效\n            "interval": 3           # 这次睡觉和上一次起床的时间间隔，小于这个时间就不允许睡觉，睡个锤子，快起床！\n        }\n    }\n}\n``` \n\n1. 默认配置（如上）\n\n    - 早安：\n\n\t\t是否要求规定时间内起床：是\n\n\t\t是否允许连续多次起床：否\n\n\t\t是否允许超级亢奋(即睡眠时长很短)：否\n\n    - 晚安：\n\n\t\t是否要求规定时间内睡觉：是\n\n\t\t是否开启优质睡眠：是\n      \n\t\t是否允许深度睡眠(即清醒时长很短)：否\n\n2. 早安配置\n    \n    - [早安开启 xx] 开启某个配置选项，配置项有：时限/多重起床/超级亢奋；例如，[早安开启 多重起床]；\n    \n    - [早安关闭 xx] 关闭某个配置选项，配置项有：时限/多重起床/超级亢奋；例如，[早安关闭 时限]；\n    \n    - [早安设置 xx x] 设置某个配置的参数，配置项有：时限/多重起床/超级亢奋；（x可选值为0到24的整数）\n      \n\t\t⚠ 配置参数时，可一次性输入规定的参数，也可通过引导一步步设置\n\t  \n\t\t⚠ 当**设置时限**时需要两个参数，命令为：[早安设置 时限 x y]，其余只需一个参数，例如，[早安设置 超级亢奋 5]\n\n\t\t⚠ 配置项具体含义参见上述规则配置\n\t\t\n\t\t⚠ 当配置某个选项时，会自动启用该选项\n\n3. 晚安配置\n    \n    - [晚安开启 xx] 开启某个配置选项，配置项有：时限/优质睡眠/深度睡眠；例如，[晚安开启 优质睡眠]；\n    \n    - [晚安关闭 xx] 关闭某个配置选项，配置项有：时限/优质睡眠/深度睡眠；例如，[晚安关闭 深度睡眠]；\n    \n    - [晚安设置 xx x] 设置某个配置的参数，配置项有：时限/优质睡眠/深度睡眠；（x可选值为0到24的整数）\n      \n\t\t⚠ 注意事项参考早安配置\n\n## 本插件改自\n\n[hoshinobot-good_morning](https://github.com/azmiao/good_morning)',
    'author': 'KafCoppelia',
    'author_email': 'k740677208@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import Coroutine, Any, List
from nonebot import logger, require, on_command, on_regex
from nonebot.matcher import Matcher
from nonebot.permission import SUPERUSER
from nonebot.adapters.onebot.v11 import Bot, GROUP, GROUP_OWNER, GROUP_ADMIN, Message, MessageSegment, GroupMessageEvent
from nonebot.params import Depends, CommandArg, RegexMatched, ArgStr
from .config import driver
from .data_source import morning_manager

require("nonebot_plugin_apscheduler")
from nonebot_plugin_apscheduler import scheduler

__morning_version__ = "v0.3.1"
__morning_notes__ = f'''
おはよう！ {__morning_version__}
[早安] 早安/哦哈哟/おはよう
[晚安] 晚安/哦呀斯密/おやすみ
[我的作息] 看看自己的作息
[群友作息] 看看群友的作息
[早晚安设置] 查看当前配置
------- 设置 -------
[早安开启/关闭 xx] 开启/关闭早安的某个配置
[早安设置 xx x] 设置早安配置的数值
[晚安开启/关闭 xx] 开启/关闭晚安的某个配置
[晚安设置 xx x] 设置晚安配置的数值'''.strip()

morning = on_command(cmd="早安", aliases={"哦哈哟", "おはよう"}, permission=GROUP, priority=12)
night = on_command(cmd="晚安", aliases={"哦呀斯密", "おやすみ"}, permission=GROUP, priority=12)

# routine
my_routine = on_command(cmd="我的作息", permission=GROUP, priority=12)
group_routine = on_command(cmd="群友作息", permission=GROUP, priority=12)

# setting
configure = on_command(cmd="早安设置", aliases={"晚安设置", "早晚安设置"}, permission=GROUP, priority=11, block=True)
morning_setting = on_regex(pattern=r"^早安(开启|关闭|设置)( (时限|多重起床|超级亢奋)(( \d{1,2}){1,2})?)?$", permission=SUPERUSER | GROUP_OWNER | GROUP_ADMIN, priority=10, block=True)
night_setting = on_regex(pattern=r"^晚安(开启|关闭|设置)( (时限|优质睡眠|深度睡眠)(( \d{1,2}){1,2})?)?$", permission=SUPERUSER | GROUP_OWNER | GROUP_ADMIN, priority=10, block=True)

@morning.handle()
async def good_morning(bot: Bot, matcher: Matcher, event: GroupMessageEvent, args: Message = CommandArg()):
    args = args.extract_plain_text()
    if args == "帮助":
        await matcher.finish(__morning_notes__)
            
    uid = event.user_id
    gid = event.group_id
    mem_info = await bot.call_api("get_group_member_info", group_id=gid, user_id=uid)
    
    sex = mem_info["sex"]
    if sex == "male":
        sex_str = "少年"
    elif sex == "female":
        sex_str = "少女"
    else:
        sex_str = "群友"

    msg = morning_manager.get_morning_msg(str(gid), str(uid), sex_str)
    await matcher.finish(message=msg, at_sender=True)

@night.handle()
async def good_night(bot: Bot, matcher: Matcher, event: GroupMessageEvent, args: Message = CommandArg()):
    args = args.extract_plain_text()
    if args == "帮助":
        await matcher.finish(__morning_notes__)
            
    uid: int = event.user_id
    gid: int = event.group_id
    mem_info = await bot.call_api("get_group_member_info", group_id=gid, user_id=uid)
    
    sex = mem_info["sex"]
    if sex == "male":
        sex_str = "少年"
    elif sex == "female":
        sex_str = "少女"
    else:
        sex_str = "群友"

    msg = morning_manager.get_night_msg(str(gid), str(uid), sex_str)
    await matcher.finish(message=msg, at_sender=True)

@my_routine.handle()
async def _(matcher: Matcher, event: GroupMessageEvent):
    gid = str(event.group_id)
    uid = str(event.user_id)
    
    msg = morning_manager.get_my_routine(gid, uid)
    await matcher.finish(message=msg, at_sender=True)

@group_routine.handle()
async def _(bot: Bot, matcher: Matcher, event: GroupMessageEvent):
    gid = event.group_id
    morning_count, night_count, uid = morning_manager.get_group_routine(str(gid))
    msg: str = f"今天已经有{morning_count}位群友早安了，{night_count}位群友晚安了~"
    
    if uid:
        mem_info = await bot.call_api("get_group_member_info", group_id=gid, user_id=int(uid))
        nickname: str = mem_info["card"] if mem_info["card"] else mem_info["nickname"]
        msg += f"\n上周睡觉大王是群友：{nickname}，再接再厉～"
    
    await matcher.finish(MessageSegment.text(msg))

@configure.handle()
async def _(matcher: Matcher):
    msg = morning_manager.get_current_config()
    await matcher.finish(msg)

def parse_item(_key: str) -> Coroutine[Any, Any, None]:
    '''
        Parser setting item
    '''
    async def _item_parser(matcher: Matcher, arg: str = ArgStr("item")) -> None:
        if arg == "取消":
            await matcher.finish("操作已取消")
        
        if arg == "时限" or arg == "多重起床" or arg == "超级亢奋" or \
            arg == "优质睡眠" or arg == "深度睡眠":
            matcher.set_arg("item", arg)
        else:
            if _key == "morning":
                await matcher.reject_arg("item", "输入配置不合法，可选时限/多重起床/超级亢奋")
            else:
                await matcher.reject_arg("item", "输入配置不合法，可选时限/优质睡眠/深度睡眠")
    
    return _item_parser

def parse_params() -> Coroutine[Any, Any, None]:
    '''
        Parser extra params
    '''
    async def _params_parser(matcher: Matcher, input_args: str = ArgStr("param1")) -> None:
        args: List[str] = input_args.split()
        
        logger.info(f"check in _params_parser: {args}")
        
        if args[0] == "取消":
            await matcher.finish("操作已取消")
        
        item = matcher.get_arg("item", None)
        if not item:
            await matcher.finish("配置出错，操作已取消")
            
        if item != "时限":
            if len(args) > 1:
                await matcher.send("输入参数过多，仅取第一个参数")
            
            try:
                _ = int(args[0])
            except ValueError:
                await matcher.reject_arg("param1", "输入参数必须是纯数字")
                
            matcher.set_arg("param1", args[0])
            matcher.set_arg("param2", 0)
        else:
            if len(args) == 1:
                await matcher.reject_arg("param1", "缺少输入参数")
            else:
                try:
                    _ = int(args[0])
                    _ = int(args[1])
                except ValueError:
                    await matcher.send("输入参数必须是纯数字，请重新输入")
                
                matcher.set_arg("param1", args[0])
                matcher.set_arg("param2", args[1])

    return _params_parser

@morning_setting.handle()
async def _(matcher: Matcher, matched: str = RegexMatched()):
    args: List[str] = matched.split()
    arg_len: int = len(args)
    
    if args[0][-2:] == "开启" or args[0][-2:] == "关闭" or args[0][-2:] == "设置":
        matcher.set_arg("op_type", args[0][-2:])
        if args[0][-2:] == "开启" or args[0][-2:] == "关闭":
            matcher.set_arg("param1", 0)   # Ignore state["param1"] and state["param2"]
            matcher.set_arg("param2", 0)
    else:
        await matcher.finish("输入指令不合法，可选：开启/关闭/设置")
    
    if arg_len > 1:
        if args[1] == "时限" or args[1] == "多重起床" or args[1] == "超级亢奋":
            matcher.set_arg("item", args[1])
        else:
            await matcher.finish("输入配置不合法，可选：时限/多重起床/超级亢奋")
    
    # Params are numbers, but store in state in string
    if arg_len > 2:
        if args[1] != "时限":
            try:
                _ = int(args[2])
            except ValueError:
                await matcher.send("输入参数必须是纯数字，请重新输入")

            matcher.set_arg("param1", args[2])
            matcher.set_arg("param2", 0)
            if arg_len > 3:
                await matcher.send("输入参数过多，仅取第一个参数")
        else:
            if arg_len < 4:
                await matcher.finish("缺少输入参数，配置项【时限】需两个参数")
            else:
                if arg_len > 4:
                    await matcher.send("输入参数过多，仅取前两个参数")
                try:
                    _ = int(args[2])
                    _ = int(args[3])
                except ValueError:
                    await matcher.send("输入参数必须是纯数字，请重新输入")
                
                matcher.set_arg("param1", args[2])
                matcher.set_arg("param2", args[3])

@morning_setting.got(
    "item",
    prompt="请选择配置项，可选：时限/多重起床/超级亢奋，输入取消以取消操作",
    parameterless=[Depends(parse_item("morning"))]
)
async def handle_skip(matcher: Matcher):
    matcher.skip()

@morning_setting.got(
    "param1",
    prompt="请输入设置参数，时限配置项请输入允许的最早/晚的睡觉时间（空格间隔），其余配置项请输入一个时间，输入取消以取消操作",
    parameterless=[Depends(parse_params())]
)
async def _(matcher: Matcher):
    _op_type: str = matcher.get_arg("op_type", None)
    _item: str = matcher.get_arg("item", None)
    _param1: str = matcher.get_arg("param1", 0)
    _param2: str = matcher.get_arg("param2", 0)
    
    if _op_type == "设置":
        msg = morning_manager.morning_config(_item, int(_param1), int(_param2))
    elif _op_type == "开启":
        msg = morning_manager.morning_switch(_item, True)
    elif _op_type == "关闭":
        msg = morning_manager.morning_switch(_item, False)
        
    await morning_setting.finish(msg)
    
@night_setting.handle()
async def _(matcher: Matcher, matched: str = RegexMatched()):
    args: List[str] = matched.split()
    arg_len: int = len(args)
    
    logger.info(f"check in handle: {args}")

    if args[0][-2:] == "开启" or args[0][-2:] == "关闭" or args[0][-2:] == "设置":
        matcher.set_arg("op_type", args[0][-2:])
        if args[0][-2:] == "开启" or args[0][-2:] == "关闭":
            matcher.set_arg("param1", 0)   # Ignore state["param1"] and state["param2"]
            matcher.set_arg("param2", 0)
    else:
        await matcher.finish("输入指令不合法，可选：开启/关闭/设置")
    
    if arg_len > 1:
        if args[1] == "时限" or args[1] == "优质睡眠" or args[1] == "深度睡眠":
            matcher.set_arg("item", args[1])
        else:
            await matcher.finish("输入配置不合法，可选：时限/优质睡眠/深度睡眠")
            
    # Params are numbers, but store in state in string
    if arg_len > 2:
        if args[1] != "时限":
            try:
                _ = int(args[2])
            except ValueError:
                await matcher.send("输入参数必须是纯数字，请重新输入")

            matcher.set_arg("param1", args[2])
            matcher.set_arg("param2", 0)
            if arg_len > 3:
                await matcher.send("输入参数过多，仅取第一个参数")
        else:
            if arg_len < 4:
                await matcher.finish("缺少输入参数，配置项【时限】需两个参数")
            else:
                if arg_len > 4:
                    await matcher.send("输入参数过多，仅取前两个参数")
                try:
                    _ = int(args[2])
                    _ = int(args[3])
                except ValueError:
                    await matcher.send("输入参数必须是纯数字，请重新输入")
                
                matcher.set_arg("param1", args[2])
                matcher.set_arg("param2", args[3])
                
@night_setting.got(
    "item",
    prompt="请选择配置项，可选：时限/优质睡眠/深度睡眠，输入取消以取消操作",
    parameterless=[Depends(parse_item("night"))]
)
async def handle_skip(matcher: Matcher):
    matcher.skip()

@night_setting.got(
    "param1",
    prompt="请输入设置参数，时限配置项请输入允许的最早/晚的睡觉时间（空格间隔），其余配置项请输入一个时间，输入取消以取消操作",
    parameterless=[Depends(parse_params())]
)
async def _(matcher: Matcher):
    _op_type: str = matcher.get_arg("op_type", None)
    _item: str = matcher.get_arg("item", None)
    _param1: str = matcher.get_arg("param1", 0)
    _param2: str = matcher.get_arg("param2", 0)
    
    if _op_type == "设置":
        msg = morning_manager.night_config(_item, int(_param1), int(_param2))
    elif _op_type == "开启":
        msg = morning_manager.night_switch(_item, True)
    elif _op_type == "关闭":
        msg = morning_manager.night_switch(_item, False)
    
    await night_setting.finish(msg)

# 每日最早晚安时间，重置昨日早晚安计数
@driver.on_startup
async def daily_refresh():
    morning_manager.daily_scheduler()
    logger.info("每日早晚安定时刷新任务已启动！")

# 每周一零点统计部分周数据
@scheduler.scheduled_job("cron", hour=0, minute=0, day_of_week="0", misfire_grace_time=60)
async def monday_refresh():
    morning_manager.weekly_night_refresh()
    logger.info("每周晚安已刷新！")

# 每周一最晚早安时间，统计上周睡眠时间、早安并重置
@driver.on_startup
async def weekly_refresh():
    morning_manager.weekly_sleep_time_scheduler()
    logger.info("每周睡眠时间定时刷新任务已启动！")


from nonebot import get_driver, logger
from pathlib import Path
from pydantic import BaseModel, Extra
from typing import Dict, List, Union
import json
from .utils import morning_json_update

class PluginConfig(BaseModel, extra=Extra.ignore):
    morning_path: Path = Path(__file__).parent / "resource"
    
mor_switcher: Dict[str, str] = {
    "时限": "morning_intime",
    "多重起床": "multi_get_up",
    "超级亢奋": "super_get_up"
}

nig_switcher: Dict[str, str] = {
    "时限": "night_intime",
    "优质睡眠": "good_sleep",
    "深度睡眠": "deep_sleep"
}

morning_prompt: List[str] = [
    "早安！",
    "おはよう！",
    "早安～",
    "哦哈哟！"
]
    
driver = get_driver()
morning_config: PluginConfig = PluginConfig.parse_obj(driver.config.dict())
        
@driver.on_startup
async def _() -> None:
    if not morning_config.morning_path.exists():
        morning_config.morning_path.mkdir(parents=True, exist_ok=True)
    
    config_json_path: Path = morning_config.morning_path / "config.json"
    # Initial value of config.json
    _config: Dict[str, Dict[str, Dict[str, Union[bool, int]]]] = {
        "morning": {
            "morning_intime": {
                "enable": True,
                "early_time": 6,
                "late_time": 12
            },
            "multi_get_up": {
                "enable": False,
                "interval": 6
            },
            "super_get_up": {
                "enable": False,
                "interval": 3
            }
        },
        "night": {
            "night_intime": {
                "enable": True,
                "early_time": 21,
                "late_time": 6
            },
            "good_sleep": {
                "enable": True,
                "interval": 6
            },
            "deep_sleep": {
                "enable": False,
                "interval": 3
            }
        }
    }
    
    if not config_json_path.exists():
        with open(config_json_path, 'w', encoding='utf-8') as f:
            json.dump(_config, f, ensure_ascii=False, indent=4)
        
        logger.info("Initialized the config.json of Morning plugin")
    else:
        # If config.json exists, transfer it if it's of old version
        with open(config_json_path, 'r', encoding='utf-8') as f:
            _c: Dict[str, Dict[str, Dict[str, Union[bool, int]]]] = json.load(f)
            if "morning_intime" not in _c["morning"] or "night_intime" not in _c["night"]:
                # Replace the old key configurations
                try:
                    _c["morning"].update({"morning_intime": _c["morning"].pop("get_up_intime")})
                    _c["night"].update({"night_intime": _c["night"].pop("sleep_intime")})
                    
                    with open(config_json_path, 'w', encoding='utf-8') as f:
                        json.dump(_c, f, ensure_ascii=False, indent=4)
                    
                    logger.info("config.json 数据格式已自动更新！")
                except KeyError:
                    # Write the initial value if error occurred
                    with open(config_json_path, 'w', encoding='utf-8') as f:
                        json.dump(_config, f, ensure_ascii=False, indent=4)
                
                    logger.info("Initialized the config.json of Morning plugin")
    
    # In version 0.3.0, old data.json will be transferred into new version if old_morning_compatible flag is enabled
    old_data_path: Path = morning_config.morning_path / "data.json"
    new_data_path: Path = morning_config.morning_path / "morning.json"

    if not new_data_path.exists():
        if old_data_path.exists():
            with open(old_data_path, 'r', encoding='utf-8') as f:
                _d: Dict[str, Dict[str, Dict[str, int]]] = json.load(f)
                _nfile = morning_json_update(_d)
                
                # Write into morning.json
                with open(new_data_path, 'w', encoding='utf-8') as f:
                    json.dump(_nfile, f, ensure_ascii=False, indent=4)
                
                logger.info("旧版数据文件已自动更新至新版，后续可关闭 OLD_MORNING_COMPATIBLE 选项！")
            
            old_data_path.unlink()
            
        else:
            with open(new_data_path, 'w', encoding='utf-8') as f:
                json.dump(dict(), f, ensure_ascii=False, indent=4)
            
            logger.warning("旧版数据文件不存在，已自动创建新版数据文件。开启 OLD_MORNING_COMPATIBLE 选项可将旧版 data.json 数据文件自动更新。")